<?php

class Am_Plugin_AffCoupons extends Am_Plugin
{
    const PLUGIN_STATUS = self::STATUS_BETA;
    const PLUGIN_REVISION = '5.5.4';

    protected $_configPrefix = 'misc.';

    const LOG_PREFIX_DEBUG = '[AffCoupons debug]. ';
    const LOG_PREFIX_ERROR = '[AffCoupons ERROR]. ';

    const MIN_CODE_LENGHT = 8;
    const PAGE_TITLE = 'Affiliate Coupons';
    const COUPON_DATA_TYPE = 'coupon-type';
    const COUPON_TYPE_REGISTRATION = 'registration';
    const COUPON_TYPE_AFFILIATE = 'affiliate';
    const COUPON_TYPE_PRODUCT = 'product #';
    const COUPON_TYPE_CATEGORY = 'category #';
    const COUPON_TYPE_MANUAL = 'manual';

    public function _initSetupForm(Am_Form_Setup $form)
    {
        $batches = $this->getDi()->couponBatchTable->getOptions();
        $products = $this->getDi()->productTable->getOptions();
        $categories = array('cat-0' => 'Any Product');
        foreach($this->getDi()->productCategoryTable->getAdminSelectOptions() as $k => $v)
            $categories['cat-' . $k] = $v;

        $emptyArr = array('' => ' -- Please Select Coupon Batch -- ');

        $form->addText('title', array('class'=>'el-wide'))
            ->setLabel('Affiliate Coupons Page Title');

        $form->addHtmlEditor('description', null, array('showInPopup' => true))
            ->setLabel('Affiliate Coupons Page Description');

        $form->addMagicSelect('batch_ids')
            ->setLabel('Automatically Shared Coupon Batchs', 'all new affiliates will automatically get coupons from these batchs')
            ->loadOptions($batches);

        $form->addInteger('coupon_length')
            ->setLabel("Minimal Coupon Code Length\n" .
                "empty = " . self::MIN_CODE_LENGHT . " symbols");

        $group = $form->addGroup()->setLabel("Additional Rules\n" .
            "to automatically give Coupons to Affiliates depending on product purchased\n" .
            "if an affiliate has this product active or active product from this category\n" .
            "he gets a coupon from selected batch");
        $sel = $group->addMagicSelect("ac_products", array('class' => 'ac_rules'));
        $gCat = $sel->addOptgroup(___('Product Categories'), array('class' => 'product_category_id'));

        foreach ($categories as $k => $v)
        {
            $group->addHidden("ac_batch_$k", array('id' => "ac_batch_$k"));

            $batchSel = new HTML_QuickForm2_Element_Select(null, array('data-id' => "$k", 'class'=>'batch-sel', 'style' => 'padding:0'));
            $batchSel->loadOptions($emptyArr + $batches);
            $batchSel->setValue($this->getConfig("ac_batch_$k", ''));
            $attr = array(
                'data-label'=> "<i>Category</i> <strong>$v</strong> " . (string)$batchSel,
            );
            $gCat->addOption($v, $k, $attr);
        }
        $gPr = $sel->addOptgroup(___('Products'), array('class' => 'product_id'));
        foreach ($products as $k => $v)
        {
            $group->addHidden("ac_batch_$k", array('id' => "ac_batch_$k"));

            $batchSel = new HTML_QuickForm2_Element_Select(null, array('data-id' => "$k", 'class'=>'batch-sel', 'style' => 'padding:0'));
            $batchSel->loadOptions($emptyArr + $batches);
            $batchSel->setValue($this->getConfig("ac_batch_$k", ''));
            $attr = array(
                'data-label'=> "<i>Product</i> <strong>$v</strong> " . (string)$batchSel,
            );
            $gPr->addOption($v, $k, $attr);
        }

        $form->addProlog(sprintf('<div class="info">
            <a href="%s" class="link">%s</a> |
            <a href="javascript:;" id="a-add_coupon_code" class="local">%s</a>
            </div>',
                 $this->getDi()->url("admin-all-aff-coupons"), ___('Display All Affiliate Coupons'),
                  ___('Add Coupon Code to Affiliates')
            ));

        $opt = '';
        foreach ($this->getDi()->couponBatchTable->getOptions() as $k => $v)
            $opt .= "<option value='$k'>$v</option>";
        $div = <<<CUT
<style>
    .magicselect-item {
        margin: 5px 0;
    }
</style>
<div id="div-add_coupon_code" style="display: none; width: 700px;">
    <div id="form-add_coupon_code">
        <strong>Do you really want to add new coupons to affiliates?</strong>
        <br><br>
        <div id="div-batch_id">
            <strong>Select Coupon Batch</strong>
            <select name="batch_id" id="batch_id">$opt</select>
        </div>
        <br>
        <div><strong>How to add coupons:</strong></div>
        <div>
            <input type="radio" name="coupon_add_type" id="coupon_add_type" value='all' checked />
            <label for="coupon_add_type">Add</label>
            <input type="text" name="coupon_add_type_all" id="coupon_add_type_all" size='3' />
            <label for="coupon_add_type">coupons for each affiliates (empty - 1 coupon)</label>
        </div>
        <div>
            <input type="radio" name="coupon_add_type" id="coupon_add_type-limit" value='limit' />
            <label for="coupon_add_type-limit">Add one coupon for each affiliate, so that each had not more than</label>
            <input type="text" name="coupon_add_type_limit" id="coupon_add_type_limit" size='3' />
            <label for="coupon_add_type-limit">coupons (empty - 1 coupon)</label>
        </div>
        <div>
            <input type="radio" name="coupon_add_type" id="coupon_add_type-count" value='count' />
            <label for="coupon_add_type-count">Add coupons for each affiliate, so that each had</label>
            <input type="text" name="coupon_add_type_count" id="coupon_add_type_count" size='3' />
            <label for="coupon_add_type-count">coupons (empty - 1 coupon)</label>
        </div>
        <div>
            <input type="radio" name="coupon_add_type" id="coupon_add_type-radd" value='rules-add' />
            <label for="coupon_add_type-radd">Add new coupons for each affiliate according to 'Additional Rules'</label>
            <input type="hidden" name="coupon_add_type_radd" id="coupon_add_type_radd" value="1" />
        </div>
        <div>
            <input type="radio" name="coupon_add_type" id="coupon_add_type-rcheck" value='rules-check' />
            <label for="coupon_add_type-rcheck">Add new coupons according to 'Additional Rules' to affiliates who don't have them already</label>
            <input type="hidden" name="coupon_add_type_rcheck" id="coupon_add_type_rcheck" value="1" />
        </div>
    </div>
    <div id="res-add_coupon_code" style="display: none"></div>
</div>
CUT;
        $form->addEpilog($div);

        $batchIds = $this->getConfig('batch_ids', false);
        $batchId = ($batchIds) ? $batchIds[0] : '';

        $rel_url = json_encode($this->getDi()->url('misc/aff-coupons',false));
        $script = <<<CUT
jQuery(function($){
    jQuery(document).on('change','.batch-sel', function(){
        id = '#ac_batch_'+jQuery(this).data('id');
        val = jQuery(this).val();
        jQuery(id).val(jQuery(this).val());
    });

    jQuery('input[name=coupon_add_type]').click(function(){
        jQuery('#div-batch_id').toggle((jQuery(this).val()!='rules-add') && (jQuery(this).val()!='rules-check'));
    });

    jQuery("#a-add_coupon_code").click(function(){
        jQuery("#batch_id").val($batchId);
        jQuery('#form-add_coupon_code').show();
        jQuery('#res-add_coupon_code').hide();
        jQuery('#res-add_coupon_code').html('');

        jQuery("#div-add_coupon_code").dialog({
            autoOpen: true
            ,width: 700
            ,buttons: {
                "OK" : function(event){
                    jQuery(event.target).closest('button').button("disable");
                    var url = $rel_url;
                    jQuery.ajax({
                        type: 'POST'
                        ,dataType: 'json'
                        ,url: url
                        ,data : {
                            batch_id : jQuery('#batch_id').val()
                            ,add_type : jQuery('input[name=coupon_add_type]:checked').val()
                            ,count : jQuery('#coupon_add_type_'+jQuery('input[name=coupon_add_type]:checked').val()).val()
                        }
                        ,success: function(data, textStatus, request)
                        {
                            res = '<div><strong>Results:</strong></div>';
                            res += '<div>Count of all affiliates: ' + data.allAffs + '</div>';
                            res += '<div>Count of changed affiliates: ' + data.changedAffs + '</div>';
                            res += '<div>Count of created coupons: ' + data.coupons + '</div>';

                            if(data.errors.length){
                                res += '<br><div><strong>Errors/Messages:</strong></div>';
                                for (i = 0; i < data.errors.length; i++){
                                    res += '<div>' + data.errors[i] + '</div>';
                                }
                            }

                            jQuery('#form-add_coupon_code').hide();
                            jQuery('#res-add_coupon_code').show();
                            jQuery('#res-add_coupon_code').html(res);
                        }
                    });
                }
                ,"Cancel" : function(){
                    jQuery(this).dialog("close");
                }
            }
            ,closeOnEscape: true
            ,title: "Add Coupons to Affiliates"
            ,modal: true
        });

    });
});
CUT;
        $form->addScript()->setScript($script);

        $form->addAdvCheckbox('debud_mode')
            ->setLabel("Debug Mode Enabled\n" .
                "write debug info to logs\nit's recommended enable it at the first time");
    }

    function onUserTabs(Am_Event_UserTabs $event)
    {
        if($page = $event->getTabs()->findOneById('aff')) {
            $page->addPage(
                array(
                    'id' => 'aff-coupons',
                    'controller' => 'admin-aff-coupons',
                    'action' => 'index',
                    'label' => ___('Affiliate Coupons'),
                    'params' => array(
                        'user_id' => $event->getUserId(),
                    ),
                )
            );
        }
    }

    public function onUserMenu(Am_Event $event)
    {
        if($page = $event->getMenu()->findOneById('aff')) {
            $page->addPage(
                array(
                    'id' => 'aff-coupons',
                    'controller' => 'aff-coupons',
                    'action' => 'index',
                    'label' => ___('Get affiliate coupons'),
                )
            );
        }
    }

    public function onUserAfterInsert(Am_Event_UserAfterInsert $event)
    {
        $user = $event->getUser();
        if(!($batchIds = $this->getConfig('batch_ids'))) {
            $this->debugLog("onUserAfterInsert - Automatically Shared Coupon Batchs is null, user #{$user->pk()}/{$user->login} was not get coupon");
            return;
        }
        if (!$user->is_affiliate) {
            $this->debugLog("onUserAfterInsert - user #{$user->pk()}/{$user->login} is not affiliate");
            return;
        }

        $cnt = $this->createNewCoupon($batchIds, $user, self::COUPON_TYPE_REGISTRATION);
        if ($cnt < 1) {
            $this->errorLog("onUserAfterInsert - Automatically Shared Coupon creating is failed for user #{$user->pk()}/{$user->login}");
            return;
        }
        $this->debugLog("onUserAfterInsert - created $cnt coupon(s)");
    }

    public function onUserBeforeDelete(Am_Event $event)
    {
        $user = $event->getUser();
        if (!$user->is_affiliate) {
            $this->debugLog("onUserBeforeDelete - user #{$user->pk()}/{$user->login} is not affiliate");
            return;
        }

        $this->getDi()->db->query("
            DELETE FROM ?_coupon
            WHERE aff_id = ?d
        ", $user->pk());

        $this->debugLog("onUserBeforeDelete - deleted all coupons for user #{$user->pk()}/{$user->login}");
    }

    public function onUserBeforeUpdate($event)
    {
        $user = $event->getUser();
        $oldUser = $event->getOldUser();
        if ($user->is_affiliate > 0 && !$oldUser->is_affiliate) // user updated -> he is affiliate now
        {
            // check Automatically Shared Coupon Batchs
            if(!($batchIds = $this->getConfig('batch_ids')))
                $this->debugLog("onUserBeforeUpdate - Automatically Shared Coupon Batchs is null, user #{$user->pk()}/{$user->login} was not get coupon");
            else
            {
                $cnt = $this->createNewCoupon($batchIds, $user, self::COUPON_TYPE_AFFILIATE);
                if ($cnt < 1)
                {
                    $this->errorLog("onUserBeforeUpdate - Automatically Shared Coupon creating is failed for user #{$user->pk()}/{$user->login}");
                    $cnt = 0;
                }
            }

            // check Additional settings
            $activePrIds = $user->getActiveProductIds();
            $cntAdditional = $this->createCouponsIfNeeded($user, $activePrIds);
            if ($cntAdditional < 1)
            {
                $this->debugLog("onUserBeforeUpdate - Additional Coupon was not created for user #{$user->pk()}/{$user->login}, active products [" . implode(',', $activePrIds) . "]");
                $cntAdditional = 0;
            }
            $cnt += $cntAdditional;
            $this->debugLog("onUserBeforeUpdate - user #{$user->pk()}/{$user->login} is affiliate now, created $cnt coupon(s)");
            return;
        }
        if (!$user->is_affiliate && $oldUser->is_affiliate > 0) // user updated -> he is NOT affiliate now
        {
            $this->getDi()->db->query("
                DELETE FROM ?_coupon
                WHERE aff_id = ?d
            ", $user->pk());

            $this->debugLog("onUserBeforeUpdate - deleted all coupons for user #{$user->pk()}/{$user->login}");
        }
    }

    public function onSubscriptionChanged(Am_Event_SubscriptionChanged $event, User $oldUser = null)
    {
        $aff = $event->getUser();
        if (!$aff->is_affiliate)
        {
            $this->debugLog("onSubscriptionChanged - user #{$aff->pk()}/{$aff->login} is not affiliate");
            return;
        }

        $added = $event->getAdded();
        if(!empty($added))
        {
            $cnt = $this->createCouponsIfNeeded($aff, $added);
            if ($cnt < 1)
                $this->debugLog("onSubscriptionChanged - Additional Coupon was not created for user #{$aff->pk()}/{$aff->login}, added products [" . implode(',', $added) ."]");
            else
                $this->debugLog("onSubscriptionChanged - created $cnt coupon(s) for user #{$aff->pk()}/{$aff->login}");
        }

        $deleted = $event->getDeleted();
        if(!empty($deleted))
        {
            $cnt = 0;
            $currentCoupons = $this->getDi()->couponTable->findByAffId($aff->pk());
            foreach ($currentCoupons as $coupon)
                foreach ($deleted as $prId)
                    if($coupon->data()->get(self::COUPON_DATA_TYPE . '.' . $prId))
                    {
                        $this->debugLog("onSubscriptionChanged - coupon {$coupon->pk()}/{$coupon->code} delete from user #{$aff->pk()}/{$aff->login}");
                        $coupon->delete();
                        $cnt++;
                    }

            $this->debugLog("onSubscriptionChanged - deleted $cnt coupon(s) for user #{$aff->pk()}/{$aff->login}");
        }
    }

    public function directAction(Am_Mvc_Request $request, Am_Mvc_Response $response, array $invokeArgs)
    {
        if(!$this->getDi()->authAdmin->getUser() || !$this->getDi()->authAdmin->getUser()->isSuper()) return;

        ini_set('memory_limit', AM_HEAVY_MEMORY_LIMIT);

        $affs = $this->getDi()->userTable->selectObjects("SELECT * FROM ?_user WHERE is_affiliate>0");
        $count = ($c = $request->getInt('count', 1)) ? $c : 1;
        $batchId = $request->getInt('batch_id');
        $cntCoupons = 0;
        $cntAffs = 0;
        $error = array();

        switch ($request->getFiltered('add_type'))
        {
            case 'rules-add':
                foreach ($affs as $aff)
                {
                    $activePrIds = $aff->getActiveProductIds();
                    $res = $this->createCouponsIfNeeded($aff, $activePrIds);
                    if ($res > 0)
                    {
                        $cntCoupons += $res;
                        $cntAffs++;
                    } else
                    {
                        $error[] = "Coupon was not created for user #{$aff->pk()}/{$aff->login}, active products [" . implode(',', $activePrIds) . "]";
                        $this->debugLog("directAction - Coupon from rules was not created for user #{$aff->pk()}/{$aff->login}, active products [" . implode(',', $activePrIds) . "]");
                    }
                }
                break;
            case 'rules-check':
                foreach ($affs as $aff)
                {
                    $activePrIds = $aff->getActiveProductIds();
                    $res = $this->createCouponsIfNeeded($aff, $activePrIds, true);
                    if ($res > 0)
                    {
                        $cntCoupons += $res;
                        $cntAffs++;
                    } else
                    {
                        $error[] = "Coupon was not created for user #{$aff->pk()}/{$aff->login}, perhaps it already has";
                        $this->debugLog("directAction - Coupon from rules was not created for user #{$aff->pk()}/{$aff->login}, active products [" . implode(',', $activePrIds) . "], perhaps it already has");
                    }
                }
                break;
            case 'all':
                foreach ($affs as $aff)
                {
                    $res = $this->createNewCoupon($batchId, $aff, self::COUPON_TYPE_MANUAL, $count);
                    if ($res > 0)
                    {
                        $cntCoupons += $res;
                        $cntAffs++;
                    } else
                    {
                        $error[] = "Coupon creating is failed for user #{$aff->pk()}/{$aff->login}";
                        $this->errorLog("add_all - coupon creating is failed for user #{$aff->pk()}/{$aff->login}");
                    }
                }
                break;
            case 'limit':
                foreach ($affs as $aff)
                {
                    $curCount = $this->getDi()->db->selectCell("
                        SELECT COUNT(*)
                        FROM ?_coupon
                        WHERE aff_id = ?d
                    ", $aff->pk());
                    if (($count - $curCount) < 1)
                    {
                        $this->debugLog("add_limit - user #{$aff->pk()}/{$aff->login} already has $curCount coupons");
                        continue;
                    }

                    $res = $this->createNewCoupon($batchId, $aff, self::COUPON_TYPE_MANUAL);
                    if ($res > 0)
                    {
                        $cntCoupons += $res;
                        $cntAffs++;
                    } else
                    {
                        $error[] = "Coupon creating is failed for user #{$aff->pk()}/{$aff->login}";
                        $this->errorLog("add_limit - coupon creating is failed for user #{$aff->pk()}/{$aff->login}");
                    }
                }
                break;
            case 'count':
                foreach ($affs as $aff)
                {
                    $curCount = $this->getDi()->db->selectCell("
                        SELECT COUNT(*)
                        FROM ?_coupon
                        WHERE aff_id = ?d
                    ", $aff->pk());
                    $needCount = $count - $curCount;
                    if ($needCount < 1)
                    {
                        $this->debugLog("add_limit - user #{$aff->pk()}/{$aff->login} already has $curCount coupons");
                        continue;
                    }

                    $res = $this->createNewCoupon($batchId, $aff, self::COUPON_TYPE_MANUAL, $needCount);
                    if ($res > 0)
                    {
                        $cntCoupons += $res;
                        $cntAffs++;
                    }else
                    {
                        $error[] = "Coupon creating is failed for user #{$aff->pk()}/{$aff->login}";
                        $this->errorLog("add_count - coupon creating is failed for user #{$aff->pk()}/{$aff->login}");
                    }
                }
                break;

            default:
                throw new Am_Exception_InternalError(self::LOG_PREFIX_ERROR . "Unknown add_type when manual adding coupon");
        }

        $response->ajaxResponse(array(
            'allAffs' => count($affs),
            'changedAffs' => $cntAffs,
            'coupons' => $cntCoupons,
            'errors' => $error
        ));
    }

    /**
     *
     * @param type $batchIds
     * @param type $aff
     * @param type $type
     * @param type $cnt
     * @param array &$coupons - generated coupons will be returned in this variable
     * @return int
     */
    public  function createNewCoupon($batchIds, $aff, $type = false, $cnt = 1, &$coupons=null)
    {
        if(!is_array($batchIds))
            $batchIds = (array)$batchIds;
        if (empty($batchIds))
        {
            $this->errorLog("batch IDs is absent");
            return -1;
        }
        if ($aff->is_affiliate < 1)
        {
            $this->errorLog("{$aff->pk()}/{$aff->login} is not affiliate");
            return -1;
        }

        $cntCreated = 0;
        $length = $this->getConfig('coupon_length', self::MIN_CODE_LENGHT);
        for ($i = 0; $i < $cnt; $i++)
            foreach ($batchIds as $batchId)
            {
                $coupon = $this->getDi()->couponRecord;
                $coupon->batch_id = $batchId;
                $coupon->code = $this->getDi()->couponTable->generateCouponCode($length, $_);
                $coupon->aff_id = $aff->pk();
                $coupon->save();
                $this->saveCouponData($coupon, $type);
                $cntCreated++;
                if(!is_null($coupons)) $coupons[] = $coupon;
                $this->debugLog("createNewCoupon - user #{$aff->pk()}/{$aff->login} was assigned to coupon #{$coupon->pk()}/{$coupon->code} at batch #$batchId");
            }
        return $cntCreated;
    }

    protected function saveCouponData(Coupon $coupon, $type)
    {
        if(is_array($type))
            $coupon->data()->set(self::COUPON_DATA_TYPE . '.' . $type['id'], $type['txt'])->update();
        else
            $coupon->data()->set(self::COUPON_DATA_TYPE, $type)->update();
    }

    protected function createCouponsIfNeeded(User $user, $newProducts, $checkPresentCoupon = false)
    {
        $cnt = 0;
        $catPrMap = $this->getDi()->productCategoryTable->getCategoryProducts();
        foreach ($newProducts as $prId)
        {
            foreach ($this->getConfig('ac_products') as $prOrCat)
            {
                $data = false;
                if($prOrCat == 'cat-0')
                    $data = array(
                        'id' => $prId,
                        'txt' => self::COUPON_TYPE_CATEGORY . 'any product'
                    );
                elseif(preg_match('/^cat-(\d+)$/', $prOrCat, $matches) && in_array($prId, $catPrMap[$matches[1]]))
                    $data = array(
                        'id' => $prId,
                        'txt' => self::COUPON_TYPE_CATEGORY . $matches[1]
                    );
                elseif($prOrCat == $prId)
                    $data = array(
                        'id' => $prId,
                        'txt' => self::COUPON_TYPE_PRODUCT . $prId
                    );

                if($data && ($batchId = $this->getConfig("ac_batch_$prOrCat", false)))
                {
                    $present = false;
                    if($checkPresentCoupon)
                    {
                        $allAffCoupons = $this->getDi()->couponTable->findBy(array('batch_id' => $batchId, 'aff_id' => $user->pk()));
                        foreach ($allAffCoupons as $coupon)
                        {
                            if($coupon->data()->get(self::COUPON_DATA_TYPE . "." . $prId))
                            {
                                $this->debugLog("createCouponsIfNeeded - user #{$user->pk()}/{$user->login} alredy has coupon {$coupon->code} from batch id#{$batchId} for product #{$prId}");
                                $present = true;
                                break;
                            }
                        }
                    }
                    if(!$present)
                        $cnt += $this->createNewCoupon($batchId, $user, $data);
                }
            }
        }
        return $cnt;
    }

    protected function debugLog($log)
    {
        if ($this->getConfig('debud_mode'))
            $this->errorLog ($log, self::LOG_PREFIX_DEBUG);
    }

    protected function errorLog($log, $prefix = self::LOG_PREFIX_ERROR)
    {
        $this->getDi()->errorLogTable->log($prefix . $log);
    }

    public function getReadme()
    {
        return <<<CUT
<strong>AffCoupons Plugin Readme</strong>

About configuring this plugin and examples of how to use it you can see <a href="http://www.amember.com/docs/Integration/AffCoupons" target="_blank" class="link">here</a>
CUT;
    }
}

class AffCouponsController extends Am_Mvc_Controller_Grid
{
    protected $plugin;
    protected $user_id;
    protected $layout = 'layout.phtml';
    protected $is_user = true;
    protected $all_users = false;
    protected $batchOptions = array();

    public function preDispatch()
    {
        $this->setParams();
        $this->plugin = $this->getDi()->plugins_misc->loadGet('aff-coupons');
        $this->batchOptions = $this->getDi()->couponBatchTable->getOptions(false);
        parent::preDispatch();
    }

    protected function setParams()
    {
        $this->getDi()->auth->requireLogin($this->getUrl());
        if (!$this->getDi()->user->is_affiliate)
            $this->_redirect('member');
        $this->user_id = $this->getDi()->auth->getUserId();
    }

    public function createGrid()
    {
        $ds = new Am_Query($this->getDi()->couponTable, 'c');
        $ds->leftJoin('?_coupon_batch', 'cb', 'cb.batch_id = c.batch_id');
        $ds->leftJoin('?_data', 'd', '(c.coupon_id = d.id AND d.table = "coupon")');
        $ds->leftJoin('?_user', 'u', 'c.aff_id = u.user_id');

        $ds->clearFields();
        $ds->addField("c.coupon_id", 'coupon_id');
        $ds->addField("c.batch_id", 'batch_id');
        $ds->addField("c.code", 'code');
        $ds->addField("c.aff_id", 'aff_id');
        $ds->addField("cb.discount", 'discount');
        $ds->addField("cb.discount_type", 'discount_type');
        $ds->addField("cb.product_ids", 'product_ids');
        $ds->addField("cb.comment", 'comment');
        $ds->addField('c.used_count', 'used_count');
        $ds->addField('cb.use_count', 'use_count');
        $ds->addField('cb.user_use_count', 'user_use_count');
        $ds->addField('u.user_id');
        $ds->addField('d.value', 'reason');
        if($this->all_users)
            $ds->addField('CONCAT(u.name_f, " ", u.name_l, " (", u.login, ")")', 'affiliate');


        if($this->all_users)
            $ds->addWhere('c.aff_id IS NOT NULL');
        else
            $ds->addWhere('c.aff_id = ?d', $this->user_id);

        $grid = new Am_Grid_Editable(
            '_aff-coupons',
            $this->plugin->getConfig('title', Am_Plugin_AffCoupons::PAGE_TITLE),
            $ds,
            $this->getRequest(),
            $this->getView(),
            $this->getDi()
        );

        if(!$this->is_user)
        {
            $grid->addField(new Am_Grid_Field('batch_id', ___('Coupons Batch'), true))
                ->setGetFunction(array($this, 'getBatchTitle'))
                ->addDecorator(new Am_Grid_Field_Decorator_Link('admin-coupons?_coupon_a=edit&_coupon_id={batch_id}', '_top'));
        }
        $grid->addField(new Am_Grid_Field('code', ___('Coupon')));
        $grid->addField(new Am_Grid_Field('product_ids', ___('Product(s)')))->setRenderFunction(array($this, 'renderProduct'));
        $grid->addField(new Am_Grid_Field('discount', ___('Discount')))->setRenderFunction(array($this, 'renderDiscount'));
        $grid->addField(new Am_Grid_Field('batch_id', ___('Conditions/Commissions'), false))->setRenderFunction(array($this, 'renderCondition'));
        if(!$this->is_user)
            $grid->addField(new Am_Grid_Field('reason', ___('Reason'), false));
        if($this->all_users)
            $grid->addField(new Am_Grid_Field('affiliate', ___('Affiliate')))
                ->addDecorator(new Am_Grid_Field_Decorator_Link($this->view->userUrl('{user_id}'), '_top'));

        $grid->addField('used_count', ___('Used'), true, 'center', array($this, 'renderUsedCount'), '5%');
        //$grid->addField('user_use_count', ___('Maximum Usage per Customer'), true, 'center',null, '5%');

        if($this->is_user)
            $grid->addCallback(Am_Grid_ReadOnly::CB_RENDER_CONTENT, array($this, 'couponRenderContent'));

        $grid->actionsClear();
        $grid->actionAdd(new Am_Grid_Action_Edit());
        if(!$this->is_user)
            $grid->actionAdd(new Am_Grid_Action_Delete());

        $grid->setForm(array($this, 'createForm'));

        return $grid;
    }

    public function getBatchTitle($record)
    {
        return $this->batchOptions[$record->batch_id];
    }

    public function couponRenderContent(& $out)
    {
        $out = sprintf('<div class="info">%s</div><br>', $this->plugin->getConfig('description')) . $out;
    }

    public function renderProduct(Coupon $coupon, $field, Am_Grid_ReadOnly $grid)
    {
        $prIds = explode(',', $coupon->$field);
        $prs = array();
        $catPrMap = $this->getDi()->productCategoryTable->getCategoryProducts();
        foreach ($prIds as $prId)
        {
            if(preg_match('/^CATEGORY-(\d+)$/', $prId, $matches))
            {
                foreach ($catPrMap[$matches[1]] as $id)
                {
                    $title = $this->getDi()->productTable->load($id)->title;
                    if(!in_array($title, $prs))
                        $prs[] = $title;
                }
            } elseif(preg_match('/^(\d+)$/', $prId, $matches))
            {
                $title = $this->getDi()->productTable->load($matches[1])->title;
                if(!in_array($title, $prs))
                    $prs[] = $title;
            }
        }
        if (empty($prs))
            $prs[] = ___('All');

        return $this->renderTd(join(', ', $prs));
    }

    public function renderCondition(Coupon $coupon, $field, Am_Grid_ReadOnly $grid)
    {
        $rules = $this->getDi()->affCommissionRuleTable->createQuery()->selectAllRecords();

        $matchRules = array();
        foreach ($rules as $rule)
        {
            $conditions = json_decode($rule->conditions, true);
            if(empty($conditions['coupon']) || empty($conditions['coupon']['used']))
                continue;

            $match = false;
            switch ($conditions['coupon']['type'])
            {
                case 'batch':
                    if($conditions['coupon']['batch_id'] == $coupon->batch_id)
                        $match = true;
                    break;
                case 'coupon':
                    if($conditions['coupon']['code'] == $coupon->code)
                        $match = true;
                    break;
                case 'any':
                        $match = true;
                    break;
            }
            if($match)
            {
                unset($conditions['coupon']);
                $rule->conditions = json_encode($conditions);
                $matchRules[] = $rule;
            }
        }
        $txt = array();
        foreach ($matchRules as $rule)
        {
            $txt[] = ___('IF') . ' ' . (($rule->getConditions()) ?
                $rule->renderConditions() : ___('use this coupon'))
                . "<br>" . ___('THEN YOU RECEIVE') . ' ' . $rule->renderCommission();
        }

        return $this->renderTd(join('<br><br>', $txt), false);
    }

    public function renderUsedCount(Coupon $coupon, $field, Am_Grid_ReadOnly $grid) {
        return sprintf("<td align='center'>%d<span style='color:#aaa'>/%s</span></td>",
            $coupon->used_count,
            ($coupon->use_count > 2000000000)  ? ___('Unlimited') : $coupon->use_count
        );
    }

    public function renderDiscount(Coupon $coupon, $field, Am_Grid_ReadOnly $grid)
    {
        if($coupon->discount_type == 'percent')
            return $this->renderTd($coupon->$field . '%');

        return $this->renderTd(Am_Currency::render($coupon->$field));
    }

    public function createForm()
    {
        $form = new Am_Form();
        $form->addText('code')
            ->setLabel('Change Coupon Code')
            ->addRule('required')
            ->addRule('callback', ___('This Coupon Code is already being used, please select another one'), array($this, 'checkCouponCode'));
        return $form;
    }

    public function checkCouponCode($code)
    {
        return !$this->getDi()->couponTable->findByCode($code);
    }
}

class AdminAffCouponsController extends AffCouponsController
{
    protected $layout = 'admin/user-layout.phtml';
    protected $is_user = false;

    protected function setParams()
    {
        $this->user_id = $this->getRequest()->getInt('user_id');
    }

    public function checkAdminPermissions(Admin $admin)
    {
        return true;
    }
}

class AdminAllAffCouponsController extends AdminAffCouponsController
{
    protected $layout = 'admin/layout.phtml';
    protected $all_users = true;

    protected function setParams()
    {
    }

    public function checkAdminPermissions(Admin $admin)
    {
        return true;
    }
}

